/*!
 * @file main.c
 * @brief Environment3 Click example
 *
 * # Description
 * This example demonstrates the use of Environment 3 click board.
 *
 * The demo application is composed of two sections :
 *
 * ## Application Init
 * Initializes the driver, sets the default configuration, and enables the gas sensor heater.
 *
 * ## Application Task
 * Reads the temperature, humidity, pressure, and gas resistance data from the sensor and 
 * displays all values on the USB UART approximately every second.
 *
 * @note
 * The temperature and humidity data don't represent the real environmental 
 * data when the heater is enabled.
 *
 * @author Stefan Filipovic
 *
 */

#include "board.h"
#include "log.h"
#include "environment3.h"

static environment3_t environment3;
static log_t logger;
static float temperature;
static float pressure;
static float humidity;
static uint32_t gas;

void application_init ( void ) 
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    environment3_cfg_t environment3_cfg;  /**< Click config object. */

    // Logger initialization.

    LOG_MAP_USB_UART( log_cfg );
    log_cfg.level = LOG_LEVEL_DEBUG;
    log_cfg.baud = 115200;
    log_init( &logger, &log_cfg );
    Delay_ms( 100 );
    log_info( &logger, " Application Init " );

    // Click initialization.

    environment3_cfg_setup( &environment3_cfg );
    ENVIRONMENT3_MAP_MIKROBUS( environment3_cfg, MIKROBUS_1 );
    err_t init_flag  = environment3_init( &environment3, &environment3_cfg );
    if ( ( init_flag == I2C_MASTER_ERROR ) || ( init_flag == SPI_MASTER_ERROR ) ) 
    {
        log_error( &logger, " Application Init Error. " );
        log_info( &logger, " Please, run program again... " );

        for ( ; ; );
    }

    if ( environment3_default_cfg ( &environment3 ) != ENVIRONMENT3_OK )
    {
        log_error( &logger, " Default Config Error. " );
        log_info( &logger, " Please, run program again... " );

        for ( ; ; );
    }
    
    environment3_enable_heater ( &environment3, ENVIRONMENT3_ENABLE );
    log_info( &logger, " Application Task " );
}

void application_task ( void ) 
{
    environment3_get_all_data ( &environment3, &temperature, &humidity, &pressure, &gas );
    log_printf( &logger, " Temperature : %.2f C\r\n", temperature );
    
    log_printf( &logger, " Humidity : %.2f %%\r\n", humidity );
    
    log_printf( &logger, " Pressure : %.3f mBar\r\n", pressure );
    
    log_printf( &logger, " Gas Resistance : %ld Ohms\r\n", gas );
    log_printf( &logger, "--------------------------------\r\n" );
}

void main ( void ) 
{
    application_init( );

    for ( ; ; ) 
    {
        application_task( );
    }
}

// ------------------------------------------------------------------------ END
